/*
 * chunk.h
 *
 *  Created on: 2010/09/29
 *      Author: shu
 */

#ifndef CHUNK_H_
#define CHUNK_H_


#include "encoder.h"
#include "sequence_index.h"
#include <stdint.h>
#include <string>

class Chunk {
public:
  static std::string GetInformationFileName(std::string filename_prefix) {
    return filename_prefix + ".inf";
  }
  static std::string GetOffsetsFileName(std::string filename_prefix) {
    return filename_prefix + ".off";
  }
  static std::string GetNamesFileName(std::string filename_prefix) {
    return filename_prefix + ".nam";
  }
  static std::string GetSequencesFileName(std::string filename_prefix) {
    return filename_prefix + ".seq";
  }
  static std::string GetIndexFileName(std::string filename_prefix) {
    return filename_prefix + ".ind";
  }

  Chunk(std::string filename_prefix);
  virtual ~Chunk();

  uint32_t GetNumberSequences() {
    return number_sequences_;
  }

  uint32_t GetConcatenatedSequenceLength() {
    return concatenated_sequences_length_;
  }

  std::string GetName(uint32_t id) {
#pragma omp critical(load_lock_)
    {
    if (names_ == NULL) {
      LoadNames(filename_prefix_);
    }
    }
    return names_[id];
  }

  uint32_t GetOffsets(uint32_t id) {
#pragma omp critical(load_lock_)
    {
    if (offsets_ == NULL) {
      LoadOffsets(filename_prefix_);
    }
    }
    return offsets_[id];
  }

  Code *GetConcatenatedSequence() {
#pragma omp critical(load_lock_)
    {
    if (concatenated_sequence_ == NULL) {
      LoadConcatenatedSequence(filename_prefix_);
    }
    }
    return concatenated_sequence_;
  }

  void ClearConcatenatedSequence() {
#pragma omp critical(load_lock_)
    {
    if (concatenated_sequence_ != NULL) {
      delete [] concatenated_sequence_;
    }
    }
  }    

  SequenceIndex *GetIndex() {
#pragma omp critical(load_lock_)
    {
    if (index_ == NULL) {
      LoadIndex(filename_prefix_);
    }
    }
    return index_;
  }

  uint32_t GetId(uint32_t position);

private:
  void LoadInfomation(std::string filename_prefix);
  void LoadOffsets(std::string filename_prefix);
  void LoadNames(std::string filename_prefix);
  void LoadConcatenatedSequence(std::string filename_prefix);
  void LoadIndex(std::string filename_prefix);

  std::string filename_prefix_;
  uint32_t number_sequences_;
  uint32_t concatenated_sequences_length_;
  std::string *names_;
  uint32_t *offsets_;
  Code *concatenated_sequence_;
  SequenceIndex *index_;
};

#endif /* CHUNK_H_ */
