/*
 * fmi_sequence_index.cpp
 *
 *  Created on: 2011/02/28
 *      Author: shu
 */

#include "fmi_sequence_index.h"
#include "fm_index.h"
#include "encoder.h"
#include "aligner_parameters.h"
#include <stdlib.h>
#include <fstream>

using namespace std;

FmiSequenceIndex::FmiSequenceIndex(const unsigned char *text, unsigned int length, unsigned int log_interval, const Encoder &encoder)
{
  Code min_regular_letter_code = encoder.GetMinRegularLetterCode();
  Code max_regular_letter_code = encoder.GetMaxRegularLetterCode();
  Code unknown_code = encoder.GetUnknownCode();

  int count = 0;
  vector<Code> seq(length);
  for (uint32_t i = 0; i < length; ++i) {
    Code c = text[length - i - 1];
    if (min_regular_letter_code <= c && c <= max_regular_letter_code) {
      seq[i] = c;
      count = 0;
    } else {
      if (count < 2 && c != AlignerParameters::kSequenceEnd) {
        seq[i] = encoder.Encode('A');
        ++count;
      } else {
        seq[i] = unknown_code;
      }
    }
  }
  index_.Rebuild(&seq[0], seq.size(), log_interval);
}

FmiSequenceIndex::FmiSequenceIndex(ifstream &is)
:index_(is)
{}

FmiSequenceIndex::~FmiSequenceIndex()
{}

void FmiSequenceIndex::Search(unsigned int offset, unsigned int start, unsigned int end, unsigned char *start_character, unsigned char *end_character,
    unsigned int *start_list, unsigned int *end_list) {
  unsigned char start_c = *start_character;
  unsigned char end_c = *end_character;
  if (offset == 0) {
    for (unsigned int i = start_c; i <= end_c; ++i) {
      start_list[i] = index_.GetAlphabetOffset(i) + 1;
      end_list[i] = index_.GetAlphabetOffset(i + 1);
    }
  } else {
    unsigned int occur_length = index_.GetRange();
    unsigned int *sp_occured_list;
    unsigned int *ep_occured_list;
    sp_occured_list = (unsigned int *)malloc(occur_length * sizeof(unsigned int));
    ep_occured_list = (unsigned int *)malloc(occur_length * sizeof(unsigned int));
    if( (sp_occured_list == NULL)||(ep_occured_list == NULL) ){
      fprintf(stderr, "[ERROR] can not allocate momory: FmiSequenceIndex::Search\n");exit(1);
    }

    index_.Occur(start - 1, sp_occured_list, start_c, end_c);
    index_.Occur(end, ep_occured_list, start_c, end_c);

    for (unsigned int i = start_c; i <= end_c; ++i) {
      start = index_.GetAlphabetOffset(i) + sp_occured_list[i] + 1;
      end = index_.GetAlphabetOffset(i) + ep_occured_list[i];
      if (start <= end) {
        start_c = i;
        break;
      }
    }
    unsigned char last_c = start_c;
    for (unsigned int i = start_c; i <= end_c; ++i) {
      start_list[i] = index_.GetAlphabetOffset(i) + sp_occured_list[i] + 1;
      end_list[i] = index_.GetAlphabetOffset(i) + ep_occured_list[i];
      if (start_list[i] <= end_list[i]) {
        last_c = i;
      }
    }
    *start_character = start_c;
    *end_character = last_c;
  }
}


unsigned int FmiSequenceIndex::Locate(unsigned int position) {
  return index_.GetLength() - index_.Locate(position);
}


unsigned int FmiSequenceIndex::GetMaxAlphabet() {
  return index_.GetRange() - 1;
}

void FmiSequenceIndex::Load(ifstream &is) {
  index_.Load(is);
}
void FmiSequenceIndex::Save(ofstream &os) {
  index_.Save(os);
}
