/*
 * amino_mapping_fm.cpp
 *
 *  Created on: 2010/08/02
 *      Author: shu
 */
#include <cstdio>
#include <cstdlib>
#include "common_parameters.h"
#include "assert.h"
#include "logger.h"
#include "aligner.h"
#include "aligner_parameters.h"
#include "db_creator.h"
#include "db_creator_parameters.h"

#ifdef PROFILE
#include <google/profiler.h>
#endif /* PROFILE */

using namespace std;

void PrintUsage() {
  Logger *logger = Logger::GetInstance();
  logger->Log(string("GHOSTX - homology search tool. version " + CommonParameters::kVersion));
  logger->Log("\n"
              "Command and Options\n"
              "db: convert a FASTA file to GHOSTX format database files\n"
              "\n"
              "  ghostx db [-i dbFastaFile] [-o dbName] [-l chunkSize]\n"
              "\n"
              "  Options:\n"
              "  (Required)\n"
              "    -i STR    Protein sequences in FASTA format for a database\n"
              "    -o STR    The name of database\n"
              "\n"
              "  (Optional)\n"
              "    -l INT    Chunk size of the database (bytes) [1073741824 (=1GB)]\n"
              "    -t STR    Database sequence type, p (protein) or d (dna) [p]\n"
              "\n"
              "\n"
              "aln:  Search homologues of queries from database\n"
              "\n"
              "  ghostx aln [-i queries] [-o output] [-d databes] [-v maxNumAliSub]\n"
              "             [-b maxNumAliQue] [-M scoreMatrix] [-G openGap] [-E extendGap]\n"
              "             [-l CandidatesSize] [-s lowerCutoff] [-T UpperCutoff]\n"
              "             [-S searchLength] [-q queryType] [-t databaseType]\n"
              "             [-a numThreads] [-L maxNumHits] [-w maxAliLen]\n"
              "\n"
              "  Options:\n"
              "  (Required)\n"
              "    -i STR    DNA or protein sequences in FASTA format for queries\n"
              "    -o STR    Output file\n"
              "    -d STR    database name (must be formatted)\n"
              "\n"
              "  (Optional)\n"
              "    -v INT    Maximum number of alignments for each subject [1]\n"
              "    -b INT    Maximum number of the output for a query [10]\n"
              "\n"
              "    -M STR    Score matrix file[BLOSUM62]\n"
              "    -G INT    Open gap penalty [11]\n"
              "    -E INT    Extend gap penalty [1]\n"
              "\n"
              "    -l INT    Maximun size of the candidates (Bytes) [134217728 (=128MB)]\n"
              "    -s INT    Lower limit cutoff score for seed search [4]\n"
              "    -T INT    Upper limit cutoff score for seed search [30]\n"
              "    -S INT    Maximum length of alignments in seed search [10]\n"
              "    -q STR    Query sequence type, p (protein) or d (dna) [p]\n"
              "    -t STR    Database sequence type, p (protein) or d (dna) [p]\n"
              "    -F STR    Filter query sequence, T (enable) or F (disable) [T] \n" 
              "    -a INT    The number of threads [1]\n"
              "    -L INT    Maximum number of hits [67108864]");
}

int main(int argc, char* argv[]) {
  if (argc < 2 ||
      strcmp(argv[1], "-h") == 0 ||
      strcmp(argv[1], "--help") == 0) {
    PrintUsage();
    exit(1);
  }
  printf("[GHOSTX] start ghostx. \n");fflush(stdout);
  if (strcmp(argv[1], "db") == 0) {
    try {
      DbCreatorParameters parameters;
      parameters.SetParameters(argc - 1, argv + 1);
      DbCreator db_creator;
      db_creator.Run(parameters);
    } catch (exception &e) {
      Logger *logger = Logger::GetInstance();
      logger->ErrorLog(e.what());
    }
  } else if (strcmp(argv[1], "aln") == 0) {
    printf("[GHOSTX] ALIGN mode \n");fflush(stdout);
    try {
      AlignerParameters parameters;
      parameters.SetParameters(argc -1, argv + 1);
      Aligner aligner;
      aligner.Run(parameters);
    } catch (exception &e) {
      Logger *logger = Logger::GetInstance();
      logger->ErrorLog(e.what());
    }
  } else {
    cerr << "[main] unrecognized command " << argv[1] << endl;
    return EXIT_SUCCESS;
  }
  return EXIT_SUCCESS;
}
