/*
 * dna_query.h
 *
 *  Created on: 2011/07/19
 *      Author: shu
 */

#ifndef DNA_QUERY_H_
#define DNA_QUERY_H_

#include "query.h"
#include "aligner_parameters.h"
#include "encoder.h"
#include <stdint.h>

class Sequence;

class DnaQuery: public Query {
public:
  static uint32_t GetSequenceLength(uint32_t length) {
    return length*2 + 3;
  }
  DnaQuery(const Sequence &sequence, AlignerParameters &parameters);

  // return the length of DNA
  uint32_t GetRealSequenceLength() {
    return dna_length_;
  }

  uint32_t GetRealStart(uint32_t pos) {
    if (pos <= dna_length_) {
      return pos;
    } else {
      return 2*dna_length_ + 2 - pos;
    }
  }

  uint32_t GetRealEnd(uint32_t pos) {
    if (pos <= dna_length_) {
      return pos;
    } else {
      return 2*dna_length_ + 2 - pos;
    }
  }

  SequenceIndex *GetIndex() {
    if (index_ == NULL) {
      SetIndex(sequence_, encoder_);
    }
    return index_;
  }

  KarlinParameters GetUngappedKarlinParameters() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    }
    return ungapped_karlin_parameters_;
  }

  int GetUngappedExtendNominalCutoff() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    }
    return ungapped_extend_nominal_cutoff_;
  }

  int GetGappedExtendNominalCutoff() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    }
    return gapped_extend_nominal_cutoff_;
  }

  int GetGappedExtendNominalTrigger() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    }
    return gapped_extend_nominal_trigger_;
  }

protected:
  void SetName(const Sequence &sequence);
  void SetEncodedSequence(const Sequence &sequence, const Encoder &encoder, const AlignerParameters &parameters);
  void SetIndex(const std::vector<Code> &sequence, const Encoder &encoder);
  void SetStaticalParameters(const std::vector<Code> &sequence, const AlignerParameters &parameters);

private:
  AlignerParameters *parameters_;
  Encoder encoder_;
  uint32_t dna_length_;
  bool is_set_statical_parameters_;
};

#endif /* DNA_QUERY_H_ */
