/*
 * translated_dna_query.h
 *
 *  Created on: 2011/04/12
 *      Author: shu
 */

#ifndef TRANSLATED_DNA_QUERY_H_
#define TRANSLATED_DNA_QUERY_H_

#include "query.h"
#include "translator.h"
#include "sequence.h"
#include "protein_sequence.h"
#include "aligner_parameters.h"
#include <string>
#include <vector>

#include <iostream>


class TranslatedDnaQuery : public Query {
public:
  static uint32_t GetSequenceLength(uint32_t length) {
    if (length < 3) {
      return 0;
    } else {
      uint32_t t = length/3;
      return 7 + t*6;
    }
  }

  TranslatedDnaQuery(const Sequence &sequence, AlignerParameters &parameters);

  // return the max length of translated sequence
  uint32_t GetRealSequenceLength() {
    return dna_length_/3;
  }

  uint32_t GetRealStart(uint32_t pos) {
    uint32_t base_length = dna_length_/3 + 1; 
    uint32_t f = (pos - 1)/base_length;
    if (f < 3) { 
      return ((pos%base_length) - 1)*3 + f + 1;
    } else {
      return dna_length_ - (((pos%base_length) - 1)*3 + f -2) + 1;
    }
  }

  uint32_t GetRealEnd(uint32_t pos) {
    uint32_t base_length = dna_length_/3 + 1; 
    uint32_t f = (pos - 1)/base_length;
    if (f < 3) {
      return GetRealStart(pos) + 2;
    } else {
      return GetRealStart(pos) - 2;
    }
  }

  SequenceIndex *GetIndex() {
    if (index_ == NULL) {
      SetIndex(sequence_, encoder_);
    }
    return index_;
  }

  KarlinParameters GetUngappedKarlinParameters() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return ungapped_karlin_parameters_;
  }

  int GetUngappedExtendNominalCutoff() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return ungapped_extend_nominal_cutoff_;
  }

  int GetGappedExtendNominalCutoff() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return gapped_extend_nominal_cutoff_;
  }

  int GetGappedExtendNominalTrigger() {
    if (!is_set_statical_parameters_) {
      SetStaticalParameters(sequence_, *parameters_);
    } 
    return gapped_extend_nominal_trigger_;
  }

protected:
  static const uint32_t kSegWindow = 12;
  void SetName(const Sequence &sequence);
  void SetEncodedSequence(const Sequence &sequence, const Encoder &encoder, const AlignerParameters &parameters);
  void SetIndex(const std::vector<Code> &sequence, const Encoder &encoder);
  void SetStaticalParameters(const std::vector<Code> &sequence, const AlignerParameters &parameters);

private:
  AlignerParameters *parameters_;
  Encoder encoder_;
  uint32_t dna_length_;
  bool is_set_statical_parameters_;
};

#endif /* TRANSLATED_DNA_QUERY_H_ */
